#include <stdio.h>
#include <string.h>

#define MAX_CHARS_READ 512

int main(int argc,char *argv[]) {

	int ForFlac = 0, CharRead, BufferLength;  // CharRead is for storing return value of fgetc(), that's why it's int and not char
	FILE *infile, *outfile;
	char buffer[MAX_CHARS_READ];
	char *pDelimiter;
	
	switch(argc) {
	case 3:	break;
	case 4:	if (!strcmp(argv[3], "--flac"))
			ForFlac = 1;
		break;
	default:
		fprintf(stderr, "\nNAME: apetag2vc - change output of program \"apetag\" to vorbis comments.\n\nVERSION: 0.1.0\n\nSYNOPSIS: %s infile outfile [--flac]\nIf infile/outfile is -, stdin/stdout is used.\n\nOPTIONS: --flac - for tag name \"COMMENT\" write \"DESCRIPTION\" instead.\n\nEXAMPLES:\napetag -i \"ACDC - Moneytalks.mpc\" | apetag2vc - vc.txt\napetag -i \"ACDC - Moneytalks.ape\" | apetag2vc - - --flac | metaflac --import-vc-from=- \"ACDC - Moneytalks.flac\"\n\n", argv[0]);
		return 0;
	}
	
	if (strcmp(argv[1], "-")) {
		infile = fopen(argv[1], "r");
		if (infile == NULL) {
			fprintf(stderr, "Can't open input file!");
			return 1;
		}
	}
	else
		infile = stdin;
	if (strcmp(argv[2], "-")) {
		outfile = fopen(argv[2], "w");
		if (outfile == NULL) {
			fprintf(stderr, "Can't open output file for writing!");
			return 2;
		}
	}
	else
		outfile = stdout;
		
	// Transform field names to vorbis comment field names:

	do {
		CharRead = fgetc(infile);
		if (CharRead == '\"') {  // in output of program "apetag" lines containing tag info start with quotes
			fgets(buffer, MAX_CHARS_READ, infile);
			pDelimiter = strstr(buffer, "\" \"");  // pointer to the delimiter between tag name and tag value in string "buffer"
			BufferLength = strlen(buffer);
			if (!strcmp(buffer + BufferLength - 2, "\"\n")) {
				buffer[BufferLength - 2] = '\n';
				buffer[BufferLength - 1] = '\0';
			}
			if (pDelimiter != NULL) {
				pDelimiter[0] = '\0';  // now there is only tag name in buffer, tag value is behind '\0'
				if(!strcmp(buffer, "Artist"))
					fprintf(outfile, "ARTIST");

				else if (!strcmp(buffer, "Title"))
					fprintf(outfile, "TITLE");

				else if (!strcmp(buffer, "Album"))
					fprintf(outfile, "ALBUM");

				else if (!strcmp(buffer, "Genre"))
					fprintf(outfile, "GENRE");

				else if (!strcmp(buffer, "Year"))
					fprintf(outfile, "DATE");

				else if (!strcmp(buffer, "Track"))
					fprintf(outfile, "TRACKNUMBER");

				else if (!strcmp(buffer, "Comment")) {
					if (ForFlac)
						fprintf(outfile, "DESCRIPTION");  // AFAIK FLAC uses DESCRIPTION instead of COMMENT field
					else
						fprintf(outfile, "COMMENT");
				}
				else {
					fprintf(stderr, "Warning: non-standard field detected: \"%s\"\n", buffer);
					fprintf(outfile, "%s", buffer);
				}
				fprintf(outfile, "=%s", pDelimiter + 3);  // write tag value
			}
		}
	} while (CharRead != EOF);
	fclose(infile);
	fclose(outfile);
return 0;
}
